@@ -34,6 +34,7 @@
 
 int init_acl(const char *path);
 void free_acl(void);
+void clear_block_list(void);
 
 int acl_match_host(const char *ip);
 int acl_add_ip(const char *ip);
@@ -42,7 +43,9 @@ int acl_remove_ip(const char *ip);
 int get_acl_mode(void);
 
 void init_block_list();
-int check_block_list(char* addr, int err_level);
+int check_block_list(char *addr, int err_level);
 int remove_from_block_list(char *addr);
 
+int outbound_block_match_host(const char *host);
+
 #endif // _ACL_H
