@@ -50,6 +50,9 @@ struct manager_ctx {
     char **nameservers;
     int nameserver_num;
     int mtu;
+#ifdef HAVE_SETRLIMIT
+    int nofile;
+#endif
 };
 
 struct server {
